# Copyright (c) Microsoft Corporation
# Licensed under the MIT License.


class WidgetRequestResponseConstants(object):
    """Strings used to pass messages between python and javascript."""
    id = "id"
    data = "data"
    error = "error"
    request = "request"
